@@ -117,9 +117,9 @@ read_bin = function(file, what = 'raw', n = file.info(file)$size, ...) {
 #' Read files one by one, and optionally add text before/after the content. Then
 #' combine all content into one character vector.
 #' @param files A vector of file paths.
-#' @param before,after A function that takes one file path as the input and
-#'   returns values to be added before or after the content of the file.
-#'   Alternatively, they can be constant values to be added.
+#' @param before,after A function that takes one file path and its content as
+#'   the input and returns values to be added before or after the content of the
+#'   file. Alternatively, they can be constant values to be added.
 #' @return A character vector.
 #' @export
 #' @examples
@@ -132,10 +132,15 @@ read_bin = function(file, what = 'raw', n = file.info(file)$size, ...) {
 #'
 #' # add constants
 #' xfun::read_all(fs, before = '/*', after = c('*/', ''))
-read_all = function(files, before = function(f) NULL, after = function(f) NULL) {
-  b = before; a = after
+read_all = function(files, before = function(f, x) NULL, after = function(f, x) NULL) {
+  call_fun = function(fun, x, ...) {
+    if (is.function(fun)) {
+      if (length(formals(fun)) > 1) fun(x, ...) else fun(x)
+    } else fun
+  }
   x = unlist(lapply(files, function(f) {
-    c(if (is.function(b)) b(f) else b, read_utf8(f), if (is.function(a)) a(f) else a)
+    t = read_utf8(f)
+    c(call_fun(before, f, t), t, call_fun(after, f, t))
   }))
   raw_string(x)
 }
@@ -190,8 +195,7 @@ sort_file = function(..., fun = sort) {
 #' @param recursive Whether to find files recursively under a directory.
 #' @param ext A vector of filename extensions (without the leading periods).
 #' @param mimetype A regular expression to filter files based on their MIME
-#'   types, e.g., `'^text/'` for plain text files. This requires the
-#'   \pkg{mime} package.
+#'   types, e.g., `'^text/'` for plain text files.
 #' @note These functions perform in-place replacement, i.e., the files will be
 #'   overwritten. Make sure you backup your files in advance, or use version
 #'   control!
@@ -223,7 +227,7 @@ gsub_files = function(files, ...) {
 gsub_dir = function(..., dir = '.', recursive = TRUE, ext = NULL, mimetype = '.*') {
   files = list.files(dir, full.names = TRUE, recursive = recursive)
   if (length(ext)) files = files[file_ext(files) %in% ext]
-  if (mimetype != '.*') files = files[grep(mimetype, mime::guess_type(files))]
+  if (mimetype != '.*') files = files[grep(mimetype, mime_type(files))]
   gsub_files(files, ...)
 }
 
